# coding=utf-8

import requests
import time
from weixunsdkcore.signature import get_signature_headers


class WeixunRequest(object):
    """
    Weixun Request Base Class
    """
    def __init__(self, service, version=None, function=None, *args, **kwargs):
        """

        :param service:
        :param version:
        :param function:
        """
        self._service = service
        self._version = version
        self._function = function
        self._protocol_type = 'http'
        self._method_type = 'get'
        self._domain_name = ''
        self._query_params = {}
        self._body_params = {}
        self._headers = {}
        self._body = None
        self._signature = ''
        self._files = {}

    """Getters"""

    def get_service(self):
        """

        :return:
        """
        return self._service

    def get_version(self):
        """

        :return:
        """
        return self._version

    def get_function(self):
        """

        :return:
        """
        return self._function

    def get_protocol_type(self):
        """

        :return:
        """
        return self._protocol_type

    def get_method_type(self):
        """

        :return:
        """
        return self._method_type

    def get_domain_name(self):
        """
        return
        :return:
        """
        return self._domain_name

    def get_query_params(self):
        """

        :return:
        """
        self._query_params['ts'] = int(time.time())
        return self._query_params

    def get_body_params(self):
        """

        :return:
        """
        return self._body_params

    def get_headers(self):
        """

        :return:
        """
        return self._headers

    def get_files(self):
        """

        :return:
        """
        return self._files

    """Setters"""

    def set_service(self, service):
        """

        :param service:
        :return:
        """
        self._service = service

    def set_version(self, version):
        """

        :param version:
        :return:
        """
        self._version = version

    def set_function(self, function):
        """

        :param function:
        :return:
        """
        self._function = function

    def set_protocol_type(self, protocol_type):
        """

        :param protocol_type:
        :return:
        """
        self._protocol_type = protocol_type

    def set_method_type(self, method_type):
        """

        :param method_type:
        :return:
        """
        self._method_type = method_type

    def set_domain_name(self, domain_name):
        """

        :param domain_name:
        :return:
        """
        self._domain_name = domain_name

    def set_query_params(self, **query_params):
        """

        :param query_params:
        :return:
        """
        self._query_params = query_params

    def set_body_params(self, **body_params):
        """

        :param body_params:
        :return:
        """
        self._body_params = body_params

    def set_headers(self, headers):
        """

        :param headers:
        :return:
        """
        self._headers = headers

    def set_body(self, body):
        """

        :param body:
        :return:
        """
        self._body = body

    def set_files(self, files):
        """

        :param files:
        :return:
        """
        self._files = files

    """Logics"""

    def get_url(self):
        """

        :return:
        """
        return (self.get_protocol_type() +
                '://' + self.get_domain_name() +
                '/api/' + self.get_service() + '/' + self.get_version() + '/' + self.get_function() + '/')

    def add_file_param(self, name, file_path):
        """

        :param self:
        :param file_path:
        :return:
        """
        self._files[name] = open(file_path, 'rb')

    def add_query_param(self, name, value):
        """

        :param name:
        :param value:
        :return:
        """
        self._query_params[name] = value

    def add_body_param(self, name, value):
        """

        :param name:
        :param value:
        :return:
        """
        self._body_params[name] = value

    def add_header(self, name, value):
        """

        :param name:
        :param value:
        :return:
        """
        self._headers[name] = value

    def get_body(self):
        """

        :return:
        """
        if self._body:
            return self._body
        elif self.get_body_params():
            return self.get_body_params()

    def get_method_func(self):
        """

        :return:
        """
        return getattr(requests, self.get_method_type())

    def get_req(self, app_id, app_key):
        """

        :param app_id:
        :param app_key:
        :return:
        """
        # 签名
        query_params = self.get_query_params()
        body_params = self.get_body_params()
        file_params = self.get_files()
        headers = self.get_headers()
        method = self.get_method_type()
        signed_headers = get_signature_headers(query_params, body_params, app_id, app_key, headers, method)
        self.set_headers(signed_headers)
        method_func = self.get_method_func()
        req = method_func(
            url=self.get_url(),
            params=self.get_query_params(),
            headers=self.get_headers(),
            data=self.get_body(),
            files=file_params,
        )
        return req







# Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
 # distributed with this work for additional information
 # regarding copyright ownership.  The ASF licenses this file
 # to you under the Apache License, Version 2.0 (the
 # "License"); you may not use this file except in compliance
 # with the License.  You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing,
 # software distributed under the License is distributed on an
 # "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.

 #coding=utf-8

__author__ = 'alex jiang'

import hashlib
import base64
import uuid
import time
import urllib
import sys

TIME_ZONE = "GMT"
FORMAT_ISO_8601 = "%Y-%m-%dT%H:%M:%SZ"
FORMAT_RFC_2616 = "%a, %d %b %Y %X GMT"

def get_uuid():
    return str(uuid.uuid4())

def get_iso_8061_date():
    return time.strftime(FORMAT_ISO_8601, time.gmtime())

def get_rfc_2616_date():
    return time.strftime(FORMAT_RFC_2616, time.gmtime())

def md5_sum(content):
    return base64.standard_b64encode(hashlib.md5(content).digest())

def percent_encode(encodeStr):
    encodeStr = str(encodeStr)
    if sys.stdin.encoding is None:
        res = urllib.quote(encodeStr.decode('cp936').encode('utf8'), '')
    else:
        res = urllib.quote(encodeStr.decode(sys.stdin.encoding).encode('utf8'), '')
    res = res.replace('+', '%20')
    res = res.replace('*', '%2A')
    res = res.replace('%7E', '~')
    return res




from distutils.core import setup
from setuptools import find_packages

PACKAGE = "weixunsdkcore"
NAME = "WeixunSDKCore"
DESCRIPTION = "WeixunSDKCore"
AUTHOR = "Maple Liu"
AUTHOR_EMAIL = "maple.liu@microfastup.com"
URL = "http://github.com/weixunsdkcore"
VERSION = __import__(PACKAGE).__version__

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    # long_description=read("README.md"),  
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license="Apache License, Version 2.0",
    url=URL,
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
)  


#coding=utf-8

import time

class WeixunClient(object):
    """
    Weixun Service Client
    """
    def __init__(self, app_id, app_key, auto_retry=True, max_retry_time=3, user_agent=None):
        """
        初始化
        :param app_id:
        :param app_key:
        :param auto_retry:
        :param max_retry_time:
        :param user_agent:
        """
        self._app_id = app_id
        self._app_key = app_key
        self._auto_retry = auto_retry
        self._max_retry_time = max_retry_time
        self._user_agent = user_agent

    def get_app_id(self):
        """
        获取AppID
        :return:
        """
        return self._app_id

    def get_app_key(self):
        """
        获取AppKey
        :return:
        """
        return self._app_key

    def is_auto_retry(self):
        """
        :return:Boolean
        """
        return self._auto_retry

    def get_max_retry_time(self):
        """

        :return:
        """
        return self._max_retry_time

    def get_user_agent(self):
        """

        :return:
        """
        return self._user_agent

    """Setters"""

    def set_app_id(self, app_id):
        """

        :return:
        """
        self._app_id = app_id

    def set_app_key(self, app_key):
        """

        :return:
        """
        self._app_key = app_key

    def set_auto_retry(self, flag):
        """
        :return:Boolean
        """
        self._auto_retry = flag

    def set_max_retry_time(self, max_retry_time):
        """

        :return:
        """
        self._max_retry_time = max_retry_time

    def set_user_agent(self, user_agent):
        """

        :return:
        """
        self._user_agent = user_agent

    """Logics"""

    def do_request(self, request_obj):
        return request_obj.get_req(app_id=self.get_app_id(), app_key=self.get_app_key())





#coding=utf-8

import hashlib
import hmac
import base64
import helper

ACCEPT = "Accept"
CONTENT_MD5 = "Content-MD5"
CONTENT_TYPE = "Content-Type"
DATE = "Date"
QUERY_SEPARATOR = "&"
HEADER_SEPARATOR = "\n"


def refresh_sign_headers(headers):
    """
    重新封装待签名的headers
    :param headers:
    :return:
    """
    if headers is None or not isinstance(headers, dict):
        headers = dict()
    if 'Date' not in headers:
        headers["Date"] = helper.get_rfc_2616_date()
    return headers


def compose_string_to_sign(method, query_params, body_params, headers=None):
    """
    封装待签名的字符串
    :param method:
    :param query_params:
    :param body_params:
    :param headers:
    :return:
    """
    sign_to_string = ""
    sign_to_string += method
    sign_to_string += HEADER_SEPARATOR
    # if headers.has_key(DATE) and headers[DATE] is not None:
    #     sign_to_string += headers[DATE]
    sign_to_string += HEADER_SEPARATOR
    sorted_map_query = sorted(query_params.items())
    for k,v in sorted_map_query:
        sign_to_string += k
        if v:
            sign_to_string += '='
            if type(v) == unicode:
                v = v.encode('gbk')
            sign_to_string += str(v)
        sign_to_string += QUERY_SEPARATOR
    sorted_map_body = sorted(body_params.items())
    for k, v in sorted_map_body:
        sign_to_string += k
        if v:
            sign_to_string += '='
            sign_to_string += str(v)
        sign_to_string += QUERY_SEPARATOR
    if sign_to_string.endswith(QUERY_SEPARATOR):
        sign_to_string = sign_to_string[0: len(sign_to_string)-1]
    return sign_to_string


def get_signature(query_params, body_params, app_key, headers, method):
    """
    获得签名字符串
    :param query_params:
    :param app_key:
    :param headers:
    :param method:
    :return:
    """
    headers = refresh_sign_headers(headers=headers)
    sign_to_string = compose_string_to_sign(
        method=method,
        query_params=query_params,
        body_params=body_params,
        headers=headers)
    h = hmac.new(app_key, sign_to_string, hashlib.sha1)
    signature = base64.encodestring(h.digest()).strip()
    return signature


def get_signature_headers(query_params, body_params, app_id, app_key, headers, method):
    signature = get_signature(query_params, body_params, app_key, headers, method)
    headers["Authorization"] = "wxs "+app_id+":"+signature
    return headers
